from typing import Any, Dict, Type, TypeVar, Tuple, Optional, BinaryIO, TextIO, TYPE_CHECKING

from typing import List


import attr

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import Union
from dateutil.parser import isoparse
import datetime
from typing import cast, List





T = TypeVar("T", bound="OAuth20Client")

@attr.s(auto_attribs=True)
class OAuth20Client:
    """OAuth 2.0 Clients are used to perform OAuth 2.0 and OpenID Connect flows. Usually, OAuth 2.0 clients are
generated for applications which want to consume your OAuth 2.0 or OpenID Connect capabilities.

    Attributes:
        allowed_cors_origins (Union[Unset, List[str]]):
        audience (Union[Unset, List[str]]):
        authorization_code_grant_access_token_lifespan (Union[Unset, str]): Specify a time duration in milliseconds,
            seconds, minutes, hours.
        authorization_code_grant_id_token_lifespan (Union[Unset, str]): Specify a time duration in milliseconds,
            seconds, minutes, hours.
        authorization_code_grant_refresh_token_lifespan (Union[Unset, str]): Specify a time duration in milliseconds,
            seconds, minutes, hours.
        backchannel_logout_session_required (Union[Unset, bool]): OpenID Connect Back-Channel Logout Session Required

            Boolean value specifying whether the RP requires that a sid (session ID) Claim be included in the Logout
            Token to identify the RP session with the OP when the backchannel_logout_uri is used.
            If omitted, the default value is false.
        backchannel_logout_uri (Union[Unset, str]): OpenID Connect Back-Channel Logout URI

            RP URL that will cause the RP to log itself out when sent a Logout Token by the OP.
        client_credentials_grant_access_token_lifespan (Union[Unset, str]): Specify a time duration in milliseconds,
            seconds, minutes, hours.
        client_id (Union[Unset, str]): OAuth 2.0 Client ID

            The ID is autogenerated and immutable.
        client_name (Union[Unset, str]): OAuth 2.0 Client Name

            The human-readable name of the client to be presented to the
            end-user during authorization.
        client_secret (Union[Unset, str]): OAuth 2.0 Client Secret

            The secret will be included in the create request as cleartext, and then
            never again. The secret is kept in hashed format and is not recoverable once lost.
        client_secret_expires_at (Union[Unset, int]): OAuth 2.0 Client Secret Expires At

            The field is currently not supported and its value is always 0.
        client_uri (Union[Unset, str]): OAuth 2.0 Client URI

            ClientURI is a URL string of a web page providing information about the client.
            If present, the server SHOULD display this URL to the end-user in
            a clickable fashion.
        contacts (Union[Unset, List[str]]):
        created_at (Union[Unset, datetime.datetime]): OAuth 2.0 Client Creation Date

            CreatedAt returns the timestamp of the client's creation.
        frontchannel_logout_session_required (Union[Unset, bool]): OpenID Connect Front-Channel Logout Session Required

            Boolean value specifying whether the RP requires that iss (issuer) and sid (session ID) query parameters be
            included to identify the RP session with the OP when the frontchannel_logout_uri is used.
            If omitted, the default value is false.
        frontchannel_logout_uri (Union[Unset, str]): OpenID Connect Front-Channel Logout URI

            RP URL that will cause the RP to log itself out when rendered in an iframe by the OP. An iss (issuer) query
            parameter and a sid (session ID) query parameter MAY be included by the OP to enable the RP to validate the
            request and to determine which of the potentially multiple sessions is to be logged out; if either is
            included, both MUST be.
        grant_types (Union[Unset, List[str]]):
        implicit_grant_access_token_lifespan (Union[Unset, str]): Specify a time duration in milliseconds, seconds,
            minutes, hours.
        implicit_grant_id_token_lifespan (Union[Unset, str]): Specify a time duration in milliseconds, seconds, minutes,
            hours.
        jwks (Union[Unset, Any]): OAuth 2.0 Client JSON Web Key Set

            Client's JSON Web Key Set [JWK] document, passed by value. The semantics of the jwks parameter are the same as
            the jwks_uri parameter, other than that the JWK Set is passed by value, rather than by reference. This parameter
            is intended only to be used by Clients that, for some reason, are unable to use the jwks_uri parameter, for
            instance, by native applications that might not have a location to host the contents of the JWK Set. If a Client
            can use jwks_uri, it MUST NOT use jwks. One significant downside of jwks is that it does not enable key rotation
            (which jwks_uri does, as described in Section 10 of OpenID Connect Core 1.0 [OpenID.Core]). The jwks_uri and
            jwks
            parameters MUST NOT be used together.
        jwks_uri (Union[Unset, str]): OAuth 2.0 Client JSON Web Key Set URL

            URL for the Client's JSON Web Key Set [JWK] document. If the Client signs requests to the Server, it contains
            the signing key(s) the Server uses to validate signatures from the Client. The JWK Set MAY also contain the
            Client's encryption keys(s), which are used by the Server to encrypt responses to the Client. When both signing
            and encryption keys are made available, a use (Key Use) parameter value is REQUIRED for all keys in the
            referenced
            JWK Set to indicate each key's intended usage. Although some algorithms allow the same key to be used for both
            signatures and encryption, doing so is NOT RECOMMENDED, as it is less secure. The JWK x5c parameter MAY be used
            to provide X.509 representations of keys provided. When used, the bare key values MUST still be present and MUST
            match those in the certificate.
        jwt_bearer_grant_access_token_lifespan (Union[Unset, str]): Specify a time duration in milliseconds, seconds,
            minutes, hours.
        logo_uri (Union[Unset, str]): OAuth 2.0 Client Logo URI

            A URL string referencing the client's logo.
        metadata (Union[Unset, Any]):
        owner (Union[Unset, str]): OAuth 2.0 Client Owner

            Owner is a string identifying the owner of the OAuth 2.0 Client.
        policy_uri (Union[Unset, str]): OAuth 2.0 Client Policy URI

            PolicyURI is a URL string that points to a human-readable privacy policy document
            that describes how the deployment organization collects, uses,
            retains, and discloses personal data.
        post_logout_redirect_uris (Union[Unset, List[str]]):
        redirect_uris (Union[Unset, List[str]]):
        refresh_token_grant_access_token_lifespan (Union[Unset, str]): Specify a time duration in milliseconds, seconds,
            minutes, hours.
        refresh_token_grant_id_token_lifespan (Union[Unset, str]): Specify a time duration in milliseconds, seconds,
            minutes, hours.
        refresh_token_grant_refresh_token_lifespan (Union[Unset, str]): Specify a time duration in milliseconds,
            seconds, minutes, hours.
        registration_access_token (Union[Unset, str]): OpenID Connect Dynamic Client Registration Access Token

            RegistrationAccessToken can be used to update, get, or delete the OAuth2 Client. It is sent when creating a
            client
            using Dynamic Client Registration.
        registration_client_uri (Union[Unset, str]): OpenID Connect Dynamic Client Registration URL

            RegistrationClientURI is the URL used to update, get, or delete the OAuth2 Client.
        request_object_signing_alg (Union[Unset, str]): OpenID Connect Request Object Signing Algorithm

            JWS [JWS] alg algorithm [JWA] that MUST be used for signing Request Objects sent to the OP. All Request Objects
            from this Client MUST be rejected, if not signed with this algorithm.
        request_uris (Union[Unset, List[str]]):
        response_types (Union[Unset, List[str]]):
        scope (Union[Unset, str]): OAuth 2.0 Client Scope

            Scope is a string containing a space-separated list of scope values (as
            described in Section 3.3 of OAuth 2.0 [RFC6749]) that the client
            can use when requesting access tokens. Example: scope1 scope-2 scope.3 scope:4.
        sector_identifier_uri (Union[Unset, str]): OpenID Connect Sector Identifier URI

            URL using the https scheme to be used in calculating Pseudonymous Identifiers by the OP. The URL references a
            file with a single JSON array of redirect_uri values.
        subject_type (Union[Unset, str]): OpenID Connect Subject Type

            The `subject_types_supported` Discovery parameter contains a
            list of the supported subject_type values for this server. Valid types include `pairwise` and `public`.
        token_endpoint_auth_method (Union[Unset, str]): OAuth 2.0 Token Endpoint Authentication Method

            Requested Client Authentication method for the Token Endpoint. The options are:

            `client_secret_post`: (default) Send `client_id` and `client_secret` as `application/x-www-form-urlencoded` in
            the HTTP body.
            `client_secret_basic`: Send `client_id` and `client_secret` as `application/x-www-form-urlencoded` encoded in
            the HTTP Authorization header.
            `private_key_jwt`: Use JSON Web Tokens to authenticate the client.
            `none`: Used for public clients (native apps, mobile apps) which can not have secrets.
        token_endpoint_auth_signing_alg (Union[Unset, str]): OAuth 2.0 Token Endpoint Signing Algorithm

            Requested Client Authentication signing algorithm for the Token Endpoint.
        tos_uri (Union[Unset, str]): OAuth 2.0 Client Terms of Service URI

            A URL string pointing to a human-readable terms of service
            document for the client that describes a contractual relationship
            between the end-user and the client that the end-user accepts when
            authorizing the client.
        updated_at (Union[Unset, datetime.datetime]): OAuth 2.0 Client Last Update Date

            UpdatedAt returns the timestamp of the last update.
        userinfo_signed_response_alg (Union[Unset, str]): OpenID Connect Request Userinfo Signed Response Algorithm

            JWS alg algorithm [JWA] REQUIRED for signing UserInfo Responses. If this is specified, the response will be JWT
            [JWT] serialized, and signed using JWS. The default, if omitted, is for the UserInfo Response to return the
            Claims
            as a UTF-8 encoded JSON object using the application/json content-type.
    """

    allowed_cors_origins: Union[Unset, List[str]] = UNSET
    audience: Union[Unset, List[str]] = UNSET
    authorization_code_grant_access_token_lifespan: Union[Unset, str] = UNSET
    authorization_code_grant_id_token_lifespan: Union[Unset, str] = UNSET
    authorization_code_grant_refresh_token_lifespan: Union[Unset, str] = UNSET
    backchannel_logout_session_required: Union[Unset, bool] = UNSET
    backchannel_logout_uri: Union[Unset, str] = UNSET
    client_credentials_grant_access_token_lifespan: Union[Unset, str] = UNSET
    client_id: Union[Unset, str] = UNSET
    client_name: Union[Unset, str] = UNSET
    client_secret: Union[Unset, str] = UNSET
    client_secret_expires_at: Union[Unset, int] = UNSET
    client_uri: Union[Unset, str] = UNSET
    contacts: Union[Unset, List[str]] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    frontchannel_logout_session_required: Union[Unset, bool] = UNSET
    frontchannel_logout_uri: Union[Unset, str] = UNSET
    grant_types: Union[Unset, List[str]] = UNSET
    implicit_grant_access_token_lifespan: Union[Unset, str] = UNSET
    implicit_grant_id_token_lifespan: Union[Unset, str] = UNSET
    jwks: Union[Unset, Any] = UNSET
    jwks_uri: Union[Unset, str] = UNSET
    jwt_bearer_grant_access_token_lifespan: Union[Unset, str] = UNSET
    logo_uri: Union[Unset, str] = UNSET
    metadata: Union[Unset, Any] = UNSET
    owner: Union[Unset, str] = UNSET
    policy_uri: Union[Unset, str] = UNSET
    post_logout_redirect_uris: Union[Unset, List[str]] = UNSET
    redirect_uris: Union[Unset, List[str]] = UNSET
    refresh_token_grant_access_token_lifespan: Union[Unset, str] = UNSET
    refresh_token_grant_id_token_lifespan: Union[Unset, str] = UNSET
    refresh_token_grant_refresh_token_lifespan: Union[Unset, str] = UNSET
    registration_access_token: Union[Unset, str] = UNSET
    registration_client_uri: Union[Unset, str] = UNSET
    request_object_signing_alg: Union[Unset, str] = UNSET
    request_uris: Union[Unset, List[str]] = UNSET
    response_types: Union[Unset, List[str]] = UNSET
    scope: Union[Unset, str] = UNSET
    sector_identifier_uri: Union[Unset, str] = UNSET
    subject_type: Union[Unset, str] = UNSET
    token_endpoint_auth_method: Union[Unset, str] = UNSET
    token_endpoint_auth_signing_alg: Union[Unset, str] = UNSET
    tos_uri: Union[Unset, str] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    userinfo_signed_response_alg: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        allowed_cors_origins: Union[Unset, List[str]] = UNSET
        if not isinstance(self.allowed_cors_origins, Unset):
            allowed_cors_origins = self.allowed_cors_origins




        audience: Union[Unset, List[str]] = UNSET
        if not isinstance(self.audience, Unset):
            audience = self.audience




        authorization_code_grant_access_token_lifespan = self.authorization_code_grant_access_token_lifespan
        authorization_code_grant_id_token_lifespan = self.authorization_code_grant_id_token_lifespan
        authorization_code_grant_refresh_token_lifespan = self.authorization_code_grant_refresh_token_lifespan
        backchannel_logout_session_required = self.backchannel_logout_session_required
        backchannel_logout_uri = self.backchannel_logout_uri
        client_credentials_grant_access_token_lifespan = self.client_credentials_grant_access_token_lifespan
        client_id = self.client_id
        client_name = self.client_name
        client_secret = self.client_secret
        client_secret_expires_at = self.client_secret_expires_at
        client_uri = self.client_uri
        contacts: Union[Unset, List[str]] = UNSET
        if not isinstance(self.contacts, Unset):
            contacts = self.contacts




        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        frontchannel_logout_session_required = self.frontchannel_logout_session_required
        frontchannel_logout_uri = self.frontchannel_logout_uri
        grant_types: Union[Unset, List[str]] = UNSET
        if not isinstance(self.grant_types, Unset):
            grant_types = self.grant_types




        implicit_grant_access_token_lifespan = self.implicit_grant_access_token_lifespan
        implicit_grant_id_token_lifespan = self.implicit_grant_id_token_lifespan
        jwks = self.jwks
        jwks_uri = self.jwks_uri
        jwt_bearer_grant_access_token_lifespan = self.jwt_bearer_grant_access_token_lifespan
        logo_uri = self.logo_uri
        metadata = self.metadata
        owner = self.owner
        policy_uri = self.policy_uri
        post_logout_redirect_uris: Union[Unset, List[str]] = UNSET
        if not isinstance(self.post_logout_redirect_uris, Unset):
            post_logout_redirect_uris = self.post_logout_redirect_uris




        redirect_uris: Union[Unset, List[str]] = UNSET
        if not isinstance(self.redirect_uris, Unset):
            redirect_uris = self.redirect_uris




        refresh_token_grant_access_token_lifespan = self.refresh_token_grant_access_token_lifespan
        refresh_token_grant_id_token_lifespan = self.refresh_token_grant_id_token_lifespan
        refresh_token_grant_refresh_token_lifespan = self.refresh_token_grant_refresh_token_lifespan
        registration_access_token = self.registration_access_token
        registration_client_uri = self.registration_client_uri
        request_object_signing_alg = self.request_object_signing_alg
        request_uris: Union[Unset, List[str]] = UNSET
        if not isinstance(self.request_uris, Unset):
            request_uris = self.request_uris




        response_types: Union[Unset, List[str]] = UNSET
        if not isinstance(self.response_types, Unset):
            response_types = self.response_types




        scope = self.scope
        sector_identifier_uri = self.sector_identifier_uri
        subject_type = self.subject_type
        token_endpoint_auth_method = self.token_endpoint_auth_method
        token_endpoint_auth_signing_alg = self.token_endpoint_auth_signing_alg
        tos_uri = self.tos_uri
        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        userinfo_signed_response_alg = self.userinfo_signed_response_alg

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if allowed_cors_origins is not UNSET:
            field_dict["allowed_cors_origins"] = allowed_cors_origins
        if audience is not UNSET:
            field_dict["audience"] = audience
        if authorization_code_grant_access_token_lifespan is not UNSET:
            field_dict["authorization_code_grant_access_token_lifespan"] = authorization_code_grant_access_token_lifespan
        if authorization_code_grant_id_token_lifespan is not UNSET:
            field_dict["authorization_code_grant_id_token_lifespan"] = authorization_code_grant_id_token_lifespan
        if authorization_code_grant_refresh_token_lifespan is not UNSET:
            field_dict["authorization_code_grant_refresh_token_lifespan"] = authorization_code_grant_refresh_token_lifespan
        if backchannel_logout_session_required is not UNSET:
            field_dict["backchannel_logout_session_required"] = backchannel_logout_session_required
        if backchannel_logout_uri is not UNSET:
            field_dict["backchannel_logout_uri"] = backchannel_logout_uri
        if client_credentials_grant_access_token_lifespan is not UNSET:
            field_dict["client_credentials_grant_access_token_lifespan"] = client_credentials_grant_access_token_lifespan
        if client_id is not UNSET:
            field_dict["client_id"] = client_id
        if client_name is not UNSET:
            field_dict["client_name"] = client_name
        if client_secret is not UNSET:
            field_dict["client_secret"] = client_secret
        if client_secret_expires_at is not UNSET:
            field_dict["client_secret_expires_at"] = client_secret_expires_at
        if client_uri is not UNSET:
            field_dict["client_uri"] = client_uri
        if contacts is not UNSET:
            field_dict["contacts"] = contacts
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if frontchannel_logout_session_required is not UNSET:
            field_dict["frontchannel_logout_session_required"] = frontchannel_logout_session_required
        if frontchannel_logout_uri is not UNSET:
            field_dict["frontchannel_logout_uri"] = frontchannel_logout_uri
        if grant_types is not UNSET:
            field_dict["grant_types"] = grant_types
        if implicit_grant_access_token_lifespan is not UNSET:
            field_dict["implicit_grant_access_token_lifespan"] = implicit_grant_access_token_lifespan
        if implicit_grant_id_token_lifespan is not UNSET:
            field_dict["implicit_grant_id_token_lifespan"] = implicit_grant_id_token_lifespan
        if jwks is not UNSET:
            field_dict["jwks"] = jwks
        if jwks_uri is not UNSET:
            field_dict["jwks_uri"] = jwks_uri
        if jwt_bearer_grant_access_token_lifespan is not UNSET:
            field_dict["jwt_bearer_grant_access_token_lifespan"] = jwt_bearer_grant_access_token_lifespan
        if logo_uri is not UNSET:
            field_dict["logo_uri"] = logo_uri
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if owner is not UNSET:
            field_dict["owner"] = owner
        if policy_uri is not UNSET:
            field_dict["policy_uri"] = policy_uri
        if post_logout_redirect_uris is not UNSET:
            field_dict["post_logout_redirect_uris"] = post_logout_redirect_uris
        if redirect_uris is not UNSET:
            field_dict["redirect_uris"] = redirect_uris
        if refresh_token_grant_access_token_lifespan is not UNSET:
            field_dict["refresh_token_grant_access_token_lifespan"] = refresh_token_grant_access_token_lifespan
        if refresh_token_grant_id_token_lifespan is not UNSET:
            field_dict["refresh_token_grant_id_token_lifespan"] = refresh_token_grant_id_token_lifespan
        if refresh_token_grant_refresh_token_lifespan is not UNSET:
            field_dict["refresh_token_grant_refresh_token_lifespan"] = refresh_token_grant_refresh_token_lifespan
        if registration_access_token is not UNSET:
            field_dict["registration_access_token"] = registration_access_token
        if registration_client_uri is not UNSET:
            field_dict["registration_client_uri"] = registration_client_uri
        if request_object_signing_alg is not UNSET:
            field_dict["request_object_signing_alg"] = request_object_signing_alg
        if request_uris is not UNSET:
            field_dict["request_uris"] = request_uris
        if response_types is not UNSET:
            field_dict["response_types"] = response_types
        if scope is not UNSET:
            field_dict["scope"] = scope
        if sector_identifier_uri is not UNSET:
            field_dict["sector_identifier_uri"] = sector_identifier_uri
        if subject_type is not UNSET:
            field_dict["subject_type"] = subject_type
        if token_endpoint_auth_method is not UNSET:
            field_dict["token_endpoint_auth_method"] = token_endpoint_auth_method
        if token_endpoint_auth_signing_alg is not UNSET:
            field_dict["token_endpoint_auth_signing_alg"] = token_endpoint_auth_signing_alg
        if tos_uri is not UNSET:
            field_dict["tos_uri"] = tos_uri
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if userinfo_signed_response_alg is not UNSET:
            field_dict["userinfo_signed_response_alg"] = userinfo_signed_response_alg

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        _d = src_dict.copy()
        allowed_cors_origins = cast(List[str], _d.pop("allowed_cors_origins", UNSET))


        audience = cast(List[str], _d.pop("audience", UNSET))


        authorization_code_grant_access_token_lifespan = _d.pop("authorization_code_grant_access_token_lifespan", UNSET)

        authorization_code_grant_id_token_lifespan = _d.pop("authorization_code_grant_id_token_lifespan", UNSET)

        authorization_code_grant_refresh_token_lifespan = _d.pop("authorization_code_grant_refresh_token_lifespan", UNSET)

        backchannel_logout_session_required = _d.pop("backchannel_logout_session_required", UNSET)

        backchannel_logout_uri = _d.pop("backchannel_logout_uri", UNSET)

        client_credentials_grant_access_token_lifespan = _d.pop("client_credentials_grant_access_token_lifespan", UNSET)

        client_id = _d.pop("client_id", UNSET)

        client_name = _d.pop("client_name", UNSET)

        client_secret = _d.pop("client_secret", UNSET)

        client_secret_expires_at = _d.pop("client_secret_expires_at", UNSET)

        client_uri = _d.pop("client_uri", UNSET)

        contacts = cast(List[str], _d.pop("contacts", UNSET))


        _created_at = _d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at,  Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)




        frontchannel_logout_session_required = _d.pop("frontchannel_logout_session_required", UNSET)

        frontchannel_logout_uri = _d.pop("frontchannel_logout_uri", UNSET)

        grant_types = cast(List[str], _d.pop("grant_types", UNSET))


        implicit_grant_access_token_lifespan = _d.pop("implicit_grant_access_token_lifespan", UNSET)

        implicit_grant_id_token_lifespan = _d.pop("implicit_grant_id_token_lifespan", UNSET)

        jwks = _d.pop("jwks", UNSET)

        jwks_uri = _d.pop("jwks_uri", UNSET)

        jwt_bearer_grant_access_token_lifespan = _d.pop("jwt_bearer_grant_access_token_lifespan", UNSET)

        logo_uri = _d.pop("logo_uri", UNSET)

        metadata = _d.pop("metadata", UNSET)

        owner = _d.pop("owner", UNSET)

        policy_uri = _d.pop("policy_uri", UNSET)

        post_logout_redirect_uris = cast(List[str], _d.pop("post_logout_redirect_uris", UNSET))


        redirect_uris = cast(List[str], _d.pop("redirect_uris", UNSET))


        refresh_token_grant_access_token_lifespan = _d.pop("refresh_token_grant_access_token_lifespan", UNSET)

        refresh_token_grant_id_token_lifespan = _d.pop("refresh_token_grant_id_token_lifespan", UNSET)

        refresh_token_grant_refresh_token_lifespan = _d.pop("refresh_token_grant_refresh_token_lifespan", UNSET)

        registration_access_token = _d.pop("registration_access_token", UNSET)

        registration_client_uri = _d.pop("registration_client_uri", UNSET)

        request_object_signing_alg = _d.pop("request_object_signing_alg", UNSET)

        request_uris = cast(List[str], _d.pop("request_uris", UNSET))


        response_types = cast(List[str], _d.pop("response_types", UNSET))


        scope = _d.pop("scope", UNSET)

        sector_identifier_uri = _d.pop("sector_identifier_uri", UNSET)

        subject_type = _d.pop("subject_type", UNSET)

        token_endpoint_auth_method = _d.pop("token_endpoint_auth_method", UNSET)

        token_endpoint_auth_signing_alg = _d.pop("token_endpoint_auth_signing_alg", UNSET)

        tos_uri = _d.pop("tos_uri", UNSET)

        _updated_at = _d.pop("updated_at", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at,  Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)




        userinfo_signed_response_alg = _d.pop("userinfo_signed_response_alg", UNSET)

        o_auth_20_client = cls(
            allowed_cors_origins=allowed_cors_origins,
            audience=audience,
            authorization_code_grant_access_token_lifespan=authorization_code_grant_access_token_lifespan,
            authorization_code_grant_id_token_lifespan=authorization_code_grant_id_token_lifespan,
            authorization_code_grant_refresh_token_lifespan=authorization_code_grant_refresh_token_lifespan,
            backchannel_logout_session_required=backchannel_logout_session_required,
            backchannel_logout_uri=backchannel_logout_uri,
            client_credentials_grant_access_token_lifespan=client_credentials_grant_access_token_lifespan,
            client_id=client_id,
            client_name=client_name,
            client_secret=client_secret,
            client_secret_expires_at=client_secret_expires_at,
            client_uri=client_uri,
            contacts=contacts,
            created_at=created_at,
            frontchannel_logout_session_required=frontchannel_logout_session_required,
            frontchannel_logout_uri=frontchannel_logout_uri,
            grant_types=grant_types,
            implicit_grant_access_token_lifespan=implicit_grant_access_token_lifespan,
            implicit_grant_id_token_lifespan=implicit_grant_id_token_lifespan,
            jwks=jwks,
            jwks_uri=jwks_uri,
            jwt_bearer_grant_access_token_lifespan=jwt_bearer_grant_access_token_lifespan,
            logo_uri=logo_uri,
            metadata=metadata,
            owner=owner,
            policy_uri=policy_uri,
            post_logout_redirect_uris=post_logout_redirect_uris,
            redirect_uris=redirect_uris,
            refresh_token_grant_access_token_lifespan=refresh_token_grant_access_token_lifespan,
            refresh_token_grant_id_token_lifespan=refresh_token_grant_id_token_lifespan,
            refresh_token_grant_refresh_token_lifespan=refresh_token_grant_refresh_token_lifespan,
            registration_access_token=registration_access_token,
            registration_client_uri=registration_client_uri,
            request_object_signing_alg=request_object_signing_alg,
            request_uris=request_uris,
            response_types=response_types,
            scope=scope,
            sector_identifier_uri=sector_identifier_uri,
            subject_type=subject_type,
            token_endpoint_auth_method=token_endpoint_auth_method,
            token_endpoint_auth_signing_alg=token_endpoint_auth_signing_alg,
            tos_uri=tos_uri,
            updated_at=updated_at,
            userinfo_signed_response_alg=userinfo_signed_response_alg,
        )

        o_auth_20_client.additional_properties = _d
        return o_auth_20_client

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
